from pathlib import Path
from os import mkdir


def install(args):
    plugin_dir = Path('plugins')
    try:
        mkdir(plugin_dir)
    except FileExistsError:
        pass


def plugin_install_parser(plugins_subparsers):
    install_parser = plugins_subparsers.add_parser(
        'install',
        help='Manage your plugins for the bot',
    )

    install_parser.set_defaults(func=lambda __: install_parser.print_help())



from os import getenv

from bitbitbot.bot import BitBitBot
from bitbitbot.parsers import parser


def run(args):
    if args.name is None:
        print('ERROR: Missing required argument NAME')
        exit(1)
    if args.token is None:
        print('ERROR: Missing required argument TOKEN')
        exit(1)
    if args.channel is None:
        print('ERROR: Missing required argument CHANNEL')
        exit(1)

    bot = BitBitBot(
        username=args.name,
        token=args.token,
        channel=args.channel,
    )
    bot.start()


@parser
def run_parser(subparsers):
    run_parser = subparsers.add_parser(
        'run',
        help='Run the chat bot',
    )

    run_parser.add_argument(
        '-n', '--name',
        default=getenv('BITBIT_NAME'),
        dest='name',
        help='The name of the account to use for the bot. Defaults to the BITBIT_NAME environment variable',
    )
    run_parser.add_argument(
        '-t', '--token',
        default=getenv('BITBIT_TOKEN'),
        dest='token',
        help='The oauth token for the bot. Defaults to the BITBIT_TOKEN environment variable',
    )
    run_parser.add_argument(
        '-c', '--channel',
        default=getenv('BITBIT_CHANNEL'),
        dest='channel',
        help='The twitch channel the bot should join. Defaults to the BITBIT_CHANNEL environment variable',
    )
    run_parser.set_defaults(func=run)



from bitbitbot.parsers import parser

from ._install import plugin_install_parser


@parser
def plugins_parser(subparsers):
    plugin_parser = subparsers.add_parser(
        'plugins',
        help='Manage your plugins for the bot',
    )

    plugins_subparsers = plugin_parser.add_subparsers(help='subcommands')
    plugin_install_parser(plugins_subparsers)

    plugin_parser.set_defaults(func=lambda __: plugin_parser.print_help())



from collections import defaultdict

EVENTS = defaultdict(list)


def listen(event_name: str) -> callable:
    def _listener(func: callable) -> None:
        EVENTS[event_name].append(func)
    return _listener



from irc.bot import SingleServerIRCBot
from irc.client import ServerConnection, Event

from .commands import COMMANDS
from .events import EVENTS
from .models import TwitchTags


class BitBitBot(SingleServerIRCBot):
    SERVER = 'irc.chat.twitch.tv'
    PORT = 6667

    def __init__(self, username: str, token: str, channel: str):
        self.username = username
        self.channel = '#' + channel

        super().__init__(
            server_list=[
                (self.SERVER, self.PORT, token),
            ],
            nickname=self.username,
            realname=self.username,
        )

    def on_welcome(self, conn: ServerConnection, event: Event):
        conn.cap('REQ', ':twitch.tv/membership')
        conn.cap('REQ', ':twitch.tv/tags')
        conn.cap('REQ', ':twitch.tv/commands')
        conn.join(self.channel)
        print('Bot is live')

    def on_pubmsg(self, conn: ServerConnection, event: Event):
        msg = event.arguments[0]
        tags = TwitchTags.parse_obj({
            tag['key'].replace('-', '_'): tag['value']
            for tag in event.tags
        })
        if msg[0] == '!':
            command, *parts = msg[1:].split()
            if command in COMMANDS:
                COMMANDS[command](self, ' '.join(parts), tags)

        self.emit_event(
            'chat_message',
            {
                'msg': msg,
                'tags': tags,
            }
        )

    def send_message(self, message: str) -> None:
        self.connection.privmsg(self.channel, message)

    def emit_event(self, event_name: str, event_data) -> None:
        for listener in EVENTS[event_name]:
            listener(event_data)



from argparse import ArgumentParser
from pkgutil import iter_modules
from pathlib import Path
from pkgutil import walk_packages
from os import mkdir

from bitbitbot import parsers


def discover_plugins():
    plugins_path = Path('plugins')
    try:
        mkdir(plugins_path)
    except FileExistsError:
        pass

    for child in plugins_path.iterdir():
        if child.is_dir():
            mod_infos = walk_packages(
                [child.absolute()],
                '.'.join(child.parts) + '.'
            )
            for __, name, __ in mod_infos:
                __import__(name, fromlist=['_trash'])


def main():
    parser = ArgumentParser(
        prog='bitbitbot',
        description='A command line interface for the BitBitBot Twitch Chat Bot',
    )
    parser.set_defaults(func=lambda __: parser.print_help())
    subparsers = parser.add_subparsers(help='commands')

    for __, name, __ in iter_modules(parsers.__path__, f'{parsers.__name__}.'):
        __import__(name, fromlist=['_trash'])

    parsers.load_subparsers(subparsers)
    discover_plugins()

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':
    main()
